@@ -27,18 +27,11 @@
 
 #include <config.h>
 
-#include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef HAVE_STRING_H
-# include <string.h>
-#endif /* HAVE_STRING_H */
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif /* HAVE_STRINGS_H */
+#include <string.h>
 #include <fcntl.h>
 #include <limits.h>
-#include <pwd.h>
 #include <grp.h>
 
 #include "sudo_compat.h"
@@ -71,12 +64,8 @@ mysetgrent(void)
 {
     if (grf == NULL) {
 	grf = fopen(grfile, "r");
-	if (grf != NULL) {
-	    if (fcntl(fileno(grf), F_SETFD, FD_CLOEXEC) == -1) {
-		fclose(grf);
-		grf = NULL;
-	    }
-	}
+	if (grf != NULL)
+	    (void)fcntl(fileno(grf), F_SETFD, FD_CLOEXEC);
     } else {
 	rewind(grf);
     }
@@ -150,10 +139,7 @@ mygetgrnam(const char *name)
     if (grf == NULL) {
 	if ((grf = fopen(grfile, "r")) == NULL)
 	    return NULL;
-	if (fcntl(fileno(grf), F_SETFD, FD_CLOEXEC) == -1) {
-	    fclose(grf);
-	    return NULL;
-	}
+	(void)fcntl(fileno(grf), F_SETFD, FD_CLOEXEC);
     } else {
 	rewind(grf);
     }
@@ -176,10 +162,7 @@ mygetgrgid(gid_t gid)
     if (grf == NULL) {
 	if ((grf = fopen(grfile, "r")) == NULL)
 	    return NULL;
-	if (fcntl(fileno(grf), F_SETFD, FD_CLOEXEC) == -1) {
-	    fclose(grf);
-	    return NULL;
-	}
+	(void)fcntl(fileno(grf), F_SETFD, FD_CLOEXEC);
     } else {
 	rewind(grf);
     }
