@@ -29,7 +29,7 @@
 #include "sudo_util.h"
 #include "sudo_fatal.h"
 
-__dso_public int main(int argc, char *argv[]);
+sudo_dso_public int main(int argc, char *argv[]);
 
 static void
 usage(void)
