@@ -86,6 +86,17 @@ sudo_copy_file(const char *src, int src_fd, off_t src_len, const char *dst,
     ssize_t nwritten, nread;
     debug_decl(sudo_copy_file, SUDO_DEBUG_UTIL);
 
+    /* Prompt the user before zeroing out an existing file. */
+    if (dst_len > 0 && src_len == 0) {
+	fprintf(stderr, U_("%s: truncate %s to zero bytes? (y/n) [n] "),
+	    getprogname(), dst);
+	if (fgets(buf, sizeof(buf), stdin) == NULL ||
+		(buf[0] != 'y' && buf[0] != 'Y')) {
+	    sudo_warnx(U_("not overwriting %s"), dst);
+	    debug_return_int(0);
+	}
+    }
+
     /* Extend the file to the new size if larger before copying. */
     if (dst_len > 0 && src_len > dst_len) {
 	if (sudo_extend_file(dst_fd, dst, src_len) == -1)
@@ -102,24 +113,24 @@ sudo_copy_file(const char *src, int src_fd, off_t src_len, const char *dst,
 	    off += nwritten;
 	} while (nread > off);
     }
-    if (nread == 0) {
-	/* success, read to EOF */
-	if (src_len < dst_len) {
-	    /* We don't open with O_TRUNC so must truncate manually. */
-	    if (ftruncate(dst_fd, src_len) == -1) {
-		sudo_debug_printf(
-		    SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO|SUDO_DEBUG_ERRNO,
-		    "unable to truncate %s to %lld", dst, (long long)src_len);
-		goto write_error;
-	    }
-	}
-	debug_return_int(0);
-    } else if (nread < 0) {
+    if (nread == -1) {
 	sudo_warn(U_("unable to read from %s"), src);
 	debug_return_int(-1);
-    } else {
-write_error:
-	sudo_warn(U_("unable to write to %s"), dst);
-	debug_return_int(-1);
     }
+
+    /* Did the file shrink? */
+    if (src_len < dst_len) {
+	/* We don't open with O_TRUNC so must truncate manually. */
+	if (ftruncate(dst_fd, src_len) == -1) {
+	    sudo_debug_printf(
+		SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO|SUDO_DEBUG_ERRNO,
+		"unable to truncate %s to %lld", dst, (long long)src_len);
+	    goto write_error;
+	}
+    }
+
+    debug_return_int(0);
+write_error:
+    sudo_warn(U_("unable to write to %s"), dst);
+    debug_return_int(-1);
 }
