@@ -111,9 +111,9 @@ static int policy_show_version(int verbose);
 static void policy_check(int argc, char * const argv[], char *env_add[],
     char **command_info[], char **argv_out[], char **user_env_out[]);
 static void policy_list(int argc, char * const argv[],
-    int verbose, const char *list_user, char * const envp[]);
+    int verbose, const char *user, char * const envp[]);
 static void policy_validate(char * const argv[], char * const envp[]);
-static void policy_invalidate(int remove);
+static void policy_invalidate(int unlinkit);
 
 /* I/O log plugin convenience functions. */
 static void iolog_open(struct sudo_settings *settings, char * const user_info[],
@@ -226,8 +226,7 @@ main(int argc, char *argv[], char *envp[])
     sudo_warn_set_conversation(sudo_conversation);
 
     /* Load plugins. */
-    if (!sudo_load_plugins(&policy_plugin, &io_plugins, &audit_plugins,
-	    &approval_plugins))
+    if (!sudo_load_plugins())
 	sudo_fatalx("%s", U_("fatal error, unable to load plugins"));
 
     /* Allocate event base so plugin can use it. */
@@ -298,9 +297,6 @@ main(int argc, char *argv[], char *envp[])
 		SET(command_details.flags, CD_LOGIN_SHELL);
 	    if (ISSET(sudo_mode, MODE_BACKGROUND))
 		SET(command_details.flags, CD_BACKGROUND);
-	    /* Become full root (not just setuid) so user cannot kill us. */
-	    if (setuid(ROOT_UID) == -1)
-		sudo_warn("setuid(%d)", ROOT_UID);
 	    if (ISSET(command_details.flags, CD_SUDOEDIT)) {
 		status = sudo_edit(&command_details);
 	    } else {
@@ -435,9 +431,10 @@ get_user_groups(struct user_details *ud)
 	if (maxgroups < 0)
 	    maxgroups = NGROUPS_MAX;
 
+	/* Note that macOS may return ngroups > NGROUPS_MAX. */
 	if ((ud->ngroups = getgroups(0, NULL)) > 0) {
-	    /* Use groups from kernel if not too many or source is static. */
-	    if (ud->ngroups < maxgroups || group_source == GROUP_SOURCE_STATIC) {
+	    /* Use groups from kernel if not at limit or source is static. */
+	    if (ud->ngroups != maxgroups || group_source == GROUP_SOURCE_STATIC) {
 		ud->groups = reallocarray(NULL, ud->ngroups, sizeof(GETGROUPS_T));
 		if (ud->groups == NULL)
 		    goto done;
@@ -650,6 +647,7 @@ command_info_to_details(char * const info[], struct command_details *details)
     debug_decl(command_info_to_details, SUDO_DEBUG_PCOMM);
 
     memset(details, 0, sizeof(*details));
+    details->info = info;
     details->closefrom = -1;
     details->execfd = -1;
     details->flags = CD_SUDOEDIT_CHECKDIR | CD_SET_GROUPS;
@@ -1075,7 +1073,7 @@ policy_open(struct sudo_settings *settings, char * const user_info[],
 	sudo_fatalx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
 
     /*
-     * Backwards compatibility for older API versions
+     * Backward compatibility for older API versions
      */
     sudo_debug_set_active_instance(SUDO_DEBUG_INSTANCE_INITIALIZER);
     switch (policy_plugin.u.generic->version) {
@@ -1205,7 +1203,7 @@ policy_check(int argc, char * const argv[],
 
 static void
 policy_list(int argc, char * const argv[], int verbose,
-    const char *list_user, char * const envp[])
+    const char *user, char * const envp[])
 {
     const char *errstr = NULL;
     /* TODO: add list_user */
@@ -1221,7 +1219,7 @@ policy_list(int argc, char * const argv[], int verbose,
 	    policy_plugin.name);
     }
     sudo_debug_set_active_instance(policy_plugin.debug_instance);
-    ok = policy_plugin.u.policy->list(argc, argv, verbose, list_user, &errstr);
+    ok = policy_plugin.u.policy->list(argc, argv, verbose, user, &errstr);
     sudo_debug_set_active_instance(sudo_debug_instance);
 
     switch (ok) {
@@ -1294,7 +1292,7 @@ policy_validate(char * const argv[], char * const envp[])
 }
 
 static void
-policy_invalidate(int remove)
+policy_invalidate(int unlinkit)
 {
     debug_decl(policy_invalidate, SUDO_DEBUG_PCOMM);
 
@@ -1303,7 +1301,7 @@ policy_invalidate(int remove)
 	    policy_plugin.name);
     }
     sudo_debug_set_active_instance(policy_plugin.debug_instance);
-    policy_plugin.u.policy->invalidate(remove);
+    policy_plugin.u.policy->invalidate(unlinkit);
     if (policy_plugin.u.policy->version >= SUDO_API_MKVERSION(1, 15)) {
 	if (policy_plugin.u.policy->close != NULL)
 	    policy_plugin.u.policy->close(0, 0);
@@ -1339,7 +1337,7 @@ policy_init_session(struct command_details *details)
 
     if (policy_plugin.u.policy->init_session) {
 	/*
-	 * Backwards compatibility for older API versions
+	 * Backward compatibility for older API versions
 	 */
 	sudo_debug_set_active_instance(policy_plugin.debug_instance);
 	switch (policy_plugin.u.generic->version) {
@@ -1352,7 +1350,11 @@ policy_init_session(struct command_details *details)
 		&details->envp, &errstr);
 	}
 	sudo_debug_set_active_instance(sudo_debug_instance);
-	/* TODO: audit on error */
+	if (ret != 1) {
+	    audit_error(policy_plugin.name, SUDO_POLICY_PLUGIN,
+		errstr ? errstr : _("policy plugin error"),
+		details->info);
+	}
     }
 done:
     debug_return_int(ret);
@@ -1375,7 +1377,7 @@ iolog_open_int(struct plugin_container *plugin, struct sudo_settings *settings,
     }
 
     /*
-     * Backwards compatibility for older API versions
+     * Backward compatibility for older API versions
      */
     sudo_debug_set_active_instance(plugin->debug_instance);
     switch (plugin->u.generic->version) {
@@ -1627,6 +1629,47 @@ audit_show_version(int verbose)
     debug_return;
 }
 
+/*
+ * Error from plugin or front-end.
+ * The error will not be sent to plugin source, if specified.
+ */
+static void
+audit_error2(struct plugin_container *source, const char *plugin_name,
+    unsigned int plugin_type, const char *audit_msg, char * const command_info[])
+{
+    struct plugin_container *plugin;
+    const char *errstr = NULL;
+    int ok;
+    debug_decl(audit_error2, SUDO_DEBUG_PCOMM);
+
+    TAILQ_FOREACH(plugin, &audit_plugins, entries) {
+	if (plugin->u.audit->error == NULL)
+	    continue;
+
+	/* Avoid a loop if the audit plugin itself has an error. */
+	if (plugin == source)
+	    continue;
+
+	sudo_debug_set_active_instance(plugin->debug_instance);
+	ok = plugin->u.audit->error(plugin_name, plugin_type,
+	    audit_msg, command_info, &errstr);
+	sudo_debug_set_active_instance(sudo_debug_instance);
+	if (ok != 1) {
+	    /*
+	     * Don't propagate the error to other audit plugins.
+	     * It is not worth the trouble to avoid potential loops.
+	     */
+	    sudo_debug_printf(SUDO_DEBUG_ERROR,
+		"%s: plugin %s error failed, ret %d", __func__,
+		plugin->name, ok);
+	    sudo_warnx(U_("%s: unable to log error event%s%s"),
+		plugin->name, errstr ? ": " : "", errstr ? errstr : "");
+	}
+    }
+
+    debug_return;
+}
+
 /*
  * Command accepted by policy.
  * See command_info[] for additional info.
@@ -1642,7 +1685,6 @@ audit_accept(const char *plugin_name, unsigned int plugin_type,
     int ok;
     debug_decl(audit_accept, SUDO_DEBUG_PCOMM);
 
-    /* XXX - kill command if can't audit accept event */
     TAILQ_FOREACH(plugin, &audit_plugins, entries) {
 	if (plugin->u.audit->accept == NULL)
 	    continue;
@@ -1650,13 +1692,20 @@ audit_accept(const char *plugin_name, unsigned int plugin_type,
 	sudo_debug_set_active_instance(plugin->debug_instance);
 	ok = plugin->u.audit->accept(plugin_name, plugin_type,
 	    command_info, run_argv, run_envp, &errstr);
+	sudo_debug_set_active_instance(sudo_debug_instance);
 	if (ok != 1) {
-	    /* XXX - fatal error?  log error with other audit modules? */
 	    sudo_debug_printf(SUDO_DEBUG_ERROR,
 		"%s: plugin %s accept failed, ret %d", __func__,
 		plugin->name, ok);
+	    sudo_warnx(U_("%s: unable to log accept event%s%s"),
+		plugin->name, errstr ? ": " : "", errstr ? errstr : "");
+
+	    /* Notify other audit plugins and exit. */
+	    audit_error2(plugin, plugin->name, SUDO_AUDIT_PLUGIN,
+		errstr ? errstr : _("audit plugin error"), command_info);
+	    audit_close(SUDO_PLUGIN_NO_STATUS, 0);
+	    exit(EXIT_FAILURE);
 	}
-	sudo_debug_set_active_instance(sudo_debug_instance);
     }
 
     debug_return;
@@ -1681,47 +1730,31 @@ audit_reject(const char *plugin_name, unsigned int plugin_type,
 	sudo_debug_set_active_instance(plugin->debug_instance);
 	ok = plugin->u.audit->reject(plugin_name, plugin_type,
 	    audit_msg, command_info, &errstr);
+	sudo_debug_set_active_instance(sudo_debug_instance);
 	if (ok != 1) {
-	    /* TODO: notify other audit plugins of the error. */
 	    sudo_debug_printf(SUDO_DEBUG_ERROR,
 		"%s: plugin %s reject failed, ret %d", __func__,
 		plugin->name, ok);
+	    sudo_warnx(U_("%s: unable to log reject event%s%s"),
+		plugin->name, errstr ? ": " : "", errstr ? errstr : "");
+
+	    /* Notify other audit plugins. */
+	    audit_error2(plugin, plugin->name, SUDO_AUDIT_PLUGIN,
+		errstr ? errstr : _("audit plugin error"), command_info);
 	}
-	sudo_debug_set_active_instance(sudo_debug_instance);
     }
 
     debug_return;
 }
 
 /*
- * Error from policy or I/O plugin.
+ * Error from plugin or front-end.
  */
 void
 audit_error(const char *plugin_name, unsigned int plugin_type,
     const char *audit_msg, char * const command_info[])
 {
-    struct plugin_container *plugin;
-    const char *errstr = NULL;
-    int ok;
-    debug_decl(audit_error, SUDO_DEBUG_PCOMM);
-
-    TAILQ_FOREACH(plugin, &audit_plugins, entries) {
-	if (plugin->u.audit->error == NULL)
-	    continue;
-
-	sudo_debug_set_active_instance(plugin->debug_instance);
-	ok = plugin->u.audit->error(plugin_name, plugin_type,
-	    audit_msg, command_info, &errstr);
-	if (ok != 1) {
-	    /* TODO: notify other audit plugins of the error. */
-	    sudo_debug_printf(SUDO_DEBUG_ERROR,
-		"%s: plugin %s error failed, ret %d", __func__,
-		plugin->name, ok);
-	}
-	sudo_debug_set_active_instance(sudo_debug_instance);
-    }
-
-    debug_return;
+    audit_error2(NULL, plugin_name, plugin_type, audit_msg, command_info);
 }
 
 static int
